////////////////////////////////////////////////////////////////////////
// Keyboard01.c
// Mochilote - www.cpcmania.com
////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

//Enumeration to identify each physical key
typedef enum _eKey
{
	Key_CursorUp = 0,
	Key_CursorRight,
	Key_CursorDown,
	Key_F9,
	Key_F6,
	Key_F3,
	Key_Enter,
	Key_FDot,
	Key_CursorLeft, //8
	Key_Copy,
	Key_F7,
	Key_F8,
	Key_F5,
	Key_F1,
	Key_F2,
	Key_F0,
	Key_Clr, //16
	Key_BraceOpen,
	Key_Return,
	Key_BraceClose,
	Key_F4,
	Key_Shift,
	Key_BackSlash,
	Key_Control,
	Key_Caret, //24
	Key_Hyphen,
	Key_At,
	Key_P,
	Key_SemiColon,
	Key_Colon,
	Key_Slash,
	Key_Dot,
	Key_0, //32
	Key_9,
	Key_O,
	Key_I,
	Key_L,
	Key_K,
	Key_M,
	Key_Comma,
	Key_8, //40
	Key_7,
	Key_U,
	Key_Y,
	Key_H,
	Key_J,
	Key_N,
	Key_Space,
	Key_6_Joy2Up, //48
	Key_5_Joy2Down,
	Key_R_Joy2Left,
	Key_T_Joy2Right,
	Key_G_Joy2Fire,
	Key_F,
	Key_B,
	Key_V,
	Key_4, //56
	Key_3,
	Key_E,
	Key_W,
	Key_S,
	Key_D,
	Key_C,
	Key_X,
	Key_1, //64
	Key_2,
	Key_Esc,
	Key_Q,
	Key_Tab,
	Key_A,
	Key_CapsLock,
	Key_Z,
	Key_Joy1Up, //72
	Key_Joy1Down,
	Key_Joy1Left,
	Key_Joy1Right,
	Key_Joy1Fire1,
	Key_Joy1Fire2,
	Key_Joy1Fire3,
	Key_Del,
	Key_Max //80
}_ekey;

//Array to associate each physical key to an ASCII character
const unsigned char aKeyAscii[Key_Max] =
{ 
	0xF0, 0xF3, 0xF1, 0x80, 0x80, 0x80, 0x80, 0x80,
	0xF2, 0xE0, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80,
	0x10, 0x5B, 0x0D, 0x5D, 0x80, 0x80, 0x5C, 0x80,
	0x5E, 0x2D, 0x40, 0x70, 0x3B, 0x3A, 0x2F, 0x2E,
	0x30, 0x39, 0x6F, 0x69, 0x6C, 0x6B, 0x6D, 0x2C,
	0x38, 0x37, 0x75, 0x79, 0x68, 0x6A, 0x6E, 0x20,
	0x36, 0x35, 0x72, 0x74, 0x67, 0x66, 0x62, 0x76,
	0x34, 0x33, 0x65, 0x77, 0x73, 0x64, 0x63, 0x78,
	0x31, 0x32, 0x80, 0x71, 0x09, 0x61, 0x80, 0x7A,
	0x0B, 0x0A, 0x08, 0x09, 0x58, 0x5A, 0x80, 0x7F
};

#define KEYBOARD_HW_LINES 10

//Array to handle the 80 physical keys bit state
unsigned char aKeyboard[KEYBOARD_HW_LINES];


////////////////////////////////////////////////////////////////////////
//ReadKeyboard()
//Copied and adapted from the function shown in CPCWiki:
//http://www.cpcwiki.eu/index.php/Programming:Keyboard_scanning
////////////////////////////////////////////////////////////////////////
void ReadKeyboard()
{
	__asm
		di
		ld hl, #_aKeyboard
		ld bc,#0xf782
		out (c),c
		ld bc,#0xf40e
		ld e,b
		out (c),c
		ld bc,#0xf6c0
		ld d,b
		out (c),c
		ld c,#0x00
		out (c),c
		ld bc,#0xf792
		out (c),c
		ld a,#0x40
		ld c,#0x4a
		_loop:
		ld b,d
		out (c),a
		ld b,e
		ini
		inc a
		cp c
		jr c,_loop
		ld bc,#0xf782
		out (c),c
		ei
	__endasm;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//GetFirstKeyPressed
////////////////////////////////////////////////////////////////////////
enum _eKey GetFirstKeyPressed()
{
	unsigned char nKeyLine = 0;
	unsigned char nBit = 0;
	
	for(nKeyLine = 0; nKeyLine < KEYBOARD_HW_LINES; nKeyLine++)
	{
		for(nBit = 0; nBit < 8; nBit++)
		{
			if(((unsigned char)1 << nBit & aKeyboard[nKeyLine]) == 0)
				return (enum _eKey)(nKeyLine * 8 + nBit);
		}
	}
	
	return Key_Max;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//Main
////////////////////////////////////////////////////////////////////////
void main()
{
	enum _eKey eKey = Key_Max;

	while(1)
	{
		ReadKeyboard();
		eKey = GetFirstKeyPressed();
		
		if(eKey != Key_Max)
			printf("(%d, %c) %x %x %x %x %x %x %x %x %x %x\n\r", eKey, aKeyAscii[eKey], aKeyboard[0], aKeyboard[1], aKeyboard[2], aKeyboard[3], aKeyboard[4], aKeyboard[5], aKeyboard[6], aKeyboard[7], aKeyboard[8], aKeyboard[9]);
	}
}
////////////////////////////////////////////////////////////////////////