////////////////////////////////////////////////////////////////////////
// Keyboard03.c
// Mochilote - www.cpcmania.com
////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "Nick.h"

#define MAX_X 79
#define MAX_Y 199


////////////////////////////////////////////////////////////////////////
//SetColor
////////////////////////////////////////////////////////////////////////
void SetColor(unsigned char nColorIndex, unsigned char nPaletteIndex)
{
  __asm
    ld a, 4 (ix)
    ld b, 5 (ix)
    ld c, b
    call #0xBC32 ;SCR SET INK
    __endasm;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//SetPalette
////////////////////////////////////////////////////////////////////////
void SetPalette(const unsigned char *pPalette)
{
  unsigned char nColor = 0;

  for(nColor = 0; nColor < NUM_COLORS; nColor++)
    SetColor(nColor, pPalette[nColor]);
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//PutSpriteMode0
////////////////////////////////////////////////////////////////////////
void PutSpriteMode0(unsigned char *pAddress, unsigned char nWidth, unsigned char nHeight, unsigned char *pSprite)
{
	__asm
		LD L, 4(IX) 
		LD H, 5(IX) 
		LD C, 6(IX) 
		LD B, 7(IX)            
		LD E, 8(IX) 
		LD D, 9(IX) 

		_loop_alto:
		   PUSH BC
		   LD B,C
		   PUSH HL
		_loop_ancho:
		   LD A,(DE)
		   LD (HL),A
		   INC DE
		   INC HL
		   DJNZ _loop_ancho
		   POP HL
		   LD A,H
		   ADD #0x08
		   LD H,A
		   SUB #0xC0
		   JP NC, _sig_linea
		   LD BC, #0xC050
		   ADD HL,BC
		_sig_linea:
		   POP BC
		   DJNZ _loop_alto
	__endasm;
}
////////////////////////////////////////////////////////////////////////


//Enumeration to identify each physical key
typedef enum _eKey
{
	Key_CursorUp = 0,
	Key_CursorRight,
	Key_CursorDown,
	Key_F9,
	Key_F6,
	Key_F3,
	Key_Enter,
	Key_FDot,
	Key_CursorLeft, //8
	Key_Copy,
	Key_F7,
	Key_F8,
	Key_F5,
	Key_F1,
	Key_F2,
	Key_F0,
	Key_Clr, //16
	Key_BraceOpen,
	Key_Return,
	Key_BraceClose,
	Key_F4,
	Key_Shift,
	Key_BackSlash,
	Key_Control,
	Key_Caret, //24
	Key_Hyphen,
	Key_At,
	Key_P,
	Key_SemiColon,
	Key_Colon,
	Key_Slash,
	Key_Dot,
	Key_0, //32
	Key_9,
	Key_O,
	Key_I,
	Key_L,
	Key_K,
	Key_M,
	Key_Comma,
	Key_8, //40
	Key_7,
	Key_U,
	Key_Y,
	Key_H,
	Key_J,
	Key_N,
	Key_Space,
	Key_6_Joy2Up, //48
	Key_5_Joy2Down,
	Key_R_Joy2Left,
	Key_T_Joy2Right,
	Key_G_Joy2Fire,
	Key_F,
	Key_B,
	Key_V,
	Key_4, //56
	Key_3,
	Key_E,
	Key_W,
	Key_S,
	Key_D,
	Key_C,
	Key_X,
	Key_1, //64
	Key_2,
	Key_Esc,
	Key_Q,
	Key_Tab,
	Key_A,
	Key_CapsLock,
	Key_Z,
	Key_Joy1Up, //72
	Key_Joy1Down,
	Key_Joy1Left,
	Key_Joy1Right,
	Key_Joy1Fire1,
	Key_Joy1Fire2,
	Key_Joy1Fire3,
	Key_Del,
	Key_Max //80
}_ekey;


#define KEYBOARD_HW_LINES 10

//Array to handle the 80 physical keys bit state
unsigned char aKeyboard[KEYBOARD_HW_LINES];


////////////////////////////////////////////////////////////////////////
//ReadKeyboard()
//Copied and adapted from the function shown in CPCWiki:
//http://www.cpcwiki.eu/index.php/Programming:Keyboard_scanning
////////////////////////////////////////////////////////////////////////
void ReadKeyboard()
{
	__asm
		di
		ld hl, #_aKeyboard
		ld bc,#0xf782
		out (c),c
		ld bc,#0xf40e
		ld e,b
		out (c),c
		ld bc,#0xf6c0
		ld d,b
		out (c),c
		ld c,#0x00
		out (c),c
		ld bc,#0xf792
		out (c),c
		ld a,#0x40
		ld c,#0x4a
		_loop:
		ld b,d
		out (c),a
		ld b,e
		ini
		inc a
		cp c
		jr c,_loop
		ld bc,#0xf782
		out (c),c
		ei
	__endasm;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//IsKeyPressed
////////////////////////////////////////////////////////////////////////
unsigned char IsKeyPressed(enum _eKey eKey)
{
	unsigned char nKeyLine = eKey / 8;
	unsigned char nBit = eKey % 8;
	
	if(((unsigned char)1 << nBit & aKeyboard[nKeyLine]) == 0)
		return 1;
	
	return 0;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//SetMode
////////////////////////////////////////////////////////////////////////
void SetMode(unsigned char nMode)
{
	__asm
		ld a, 4 (ix)
		call #0xBC0E ;SCR_SET_MODE
	__endasm;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//Main
////////////////////////////////////////////////////////////////////////
void main()
{
	int nX = 40;
	int nY = 100;

 	SetMode(0);
 	SetPalette(NickPalette);
 
	do
	{
		ReadKeyboard();

		if(IsKeyPressed(Key_CursorUp))
			nY -= 2;

		if(IsKeyPressed(Key_CursorDown))
			nY += 2;

		if(IsKeyPressed(Key_CursorLeft))
			nX -= 1;
		
		if(IsKeyPressed(Key_CursorRight))
			nX += 1;

		if(nX <= 0)
			nX = 0;

		if(nY <= 0)
			nY = 0;
		
		if(nX >= (MAX_X - NICK_WIDTH))
			nX = MAX_X - NICK_WIDTH;

		if(nY >= (MAX_Y - NICK_HEIGHT))
			nY = MAX_Y - NICK_HEIGHT;
		
		//paint
		PutSpriteMode0((unsigned char *)(0xC000 + ((nY / 8u) * 80u) + ((nY % 8u) * 2048u) + nX), NICK_WIDTH, NICK_HEIGHT, NickSprite);
	}
	while(1);
}
////////////////////////////////////////////////////////////////////////